'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const voidjsApp = 'VOID_JS_APP';

(function (ComponentError) {
  ComponentError["functionComponentOnly"] = "Only Function Component is supported";
  ComponentError["namedFunctionComponentOnly"] = "Anonymous Function Component is not supported";
  ComponentError["reservedFunctionName"] = "Reserved Component Name";
})(exports.ComponentError || (exports.ComponentError = {}));

function index (babel) {
  const template = babel.template; // store the declaration name of default export

  let exportDefaultDeclarationName = '';
  return {
    visitor: {
      Program: {
        enter(path) {
          if (!path.scope.hasBinding('createElement')) {
            path.unshiftContainer('body', template.ast(`import { createElement } from "react";`));
          }

          if (!path.scope.hasBinding('ReactDOM')) {
            path.unshiftContainer('body', template.ast(`import ReactDOM from "react-dom";`));
          }
        },

        exit(path, state) {
          // no named default export found
          if (exportDefaultDeclarationName === '') return;
          const {
            opts: {
              root = 'app',
              hydrate = false,
              app = false
            }
          } = state;
          const h = `ReactDOM.${hydrate ? 'hydrate' : 'render'}`;
          const container = `document.getElementById("${root}")`;
          path.pushContainer('body', template.ast(`if (typeof getStaticProps !== "undefined") {
                (async function () {
                  const data = await getStaticProps();
                  if (${!!app}) {
                    import("${app}").then(({default: ${voidjsApp}}) => {
                      ${h}(createElement(${voidjsApp}, {Component: ${exportDefaultDeclarationName}, pageProps: data.props}), ${container});
                    });
                  } else {
                    ${h}(createElement(${exportDefaultDeclarationName}, data.props), ${container});
                  }
                })();
              } else {
                if (${!!app}) {
                  import("${app}").then(({default: ${voidjsApp}}) => {
                    ${h}(createElement(${voidjsApp}, {Component: ${exportDefaultDeclarationName}, pageProps: {}}), ${container});
                  });
                } else {
                  ${h}(createElement(${exportDefaultDeclarationName}), ${container});
                }
              }`));
        }

      },
      ExportDefaultDeclaration: {
        enter(path) {
          if (path.node.declaration.type !== 'FunctionDeclaration') throw new Error(exports.ComponentError.functionComponentOnly);
          if (!path.node.declaration.id) throw new Error(exports.ComponentError.namedFunctionComponentOnly);
          if (path.node.declaration.id.name === voidjsApp) throw new Error(exports.ComponentError.reservedFunctionName);
          exportDefaultDeclarationName = path.node.declaration.id.name;
        }

      }
    }
  };
}

exports.default = index;
